@@ -2,11 +2,16 @@ from .sqlalchemy_models import (
     Card,
     CardType,
     Collection,
+    Deck,
+    DeckConfig,
+    Field,
     FlagColor,
     Grave,
     GraveType,
     Note,
+    NoteType,
     QueueType,
-    ReviewwType,
     RevLog,
+    ReviewType,
+    Tag,
 )
